package com.supwisdom.dlpay

import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import org.jose4j.jwt.consumer.InvalidJwtException
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.http.HttpStatus
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.core.userdetails.User
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.provisioning.InMemoryUserDetailsManager
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter
import org.springframework.security.web.authentication.logout.CookieClearingLogoutHandler
import org.springframework.web.filter.OncePerRequestFilter
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse


class ApiJwtAuthenticationFilter(jwt: JwtTokenUtil) : OncePerRequestFilter() {
    private val jwtUtil = jwt

    override fun doFilterInternal(request: HttpServletRequest, response: HttpServletResponse, filterChain: FilterChain) {
        request.getHeader(jwtUtil.header)?.let { jwt ->
            try {
                val claims = jwtUtil.verifyToken(jwt)
                val auth = UsernamePasswordAuthenticationToken(claims["uid"], null,
                        claims["authorities"]?.map { SimpleGrantedAuthority(it as String) })
                SecurityContextHolder.getContext().authentication = auth
            } catch (e: InvalidJwtException) {
                SecurityContextHolder.clearContext();
                response.sendError(HttpStatus.BAD_REQUEST.value(), e.message)
            }
        }
        filterChain.doFilter(request, response)
    }
}

@EnableWebSecurity
class WebSecurityConfig {

    @Bean
    fun userDetailsService(): UserDetailsService {
        val manager = InMemoryUserDetailsManager()
        manager.createUser(User.withDefaultPasswordEncoder()
                .username("admin")
                .password("123456")
                .roles("USER").build())
        return manager
    }

//    @Bean
//    fun daoProvider(detailsService: UserDetailsService): DaoAuthenticationProvider {
//        return DaoAuthenticationProvider().also {
//            it.setUserDetailsService(detailsService)
//        }
//    }
//
//    @Bean
//    fun providerManager(daoProvider: DaoAuthenticationProvider): ProviderManager {
//        return ProviderManager(listOf(daoProvider))
//    }


    companion object {
        @Configuration
        @Order(1)
        class ApiWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {

            @Autowired
            lateinit var jwtConfig: JwtConfig

            override fun configure(http: HttpSecurity) {
                // 设置 API 访问权限管理
                http.csrf().disable()
                        .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                        .and()
                        .addFilterAfter(ApiJwtAuthenticationFilter(JwtTokenUtil(jwtConfig)),
                                UsernamePasswordAuthenticationFilter::class.java)
                        .authorizeRequests()
                        .antMatchers("/api/auth/**").permitAll()
                        .antMatchers("/api/common/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/consume/**").hasRole("THIRD_CONSUME")
                        .antMatchers("/api/deposit/**").hasRole("THIRD_DEPOSIT")
                        .antMatchers("/api/user/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/shop/**").hasRole("THIRD_SHOP")
                        .anyRequest().authenticated()
                // 注册 filter
            }
        }

        @Configuration
        class MvcWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {

            override fun configure(http: HttpSecurity) {
                // 设置 Web MVC 应用权限
                http.authorizeRequests()
                        .anyRequest().authenticated()
                        .and()
                        .formLogin()
                        .loginPage("/user/login").permitAll()
                        .and()
                        .logout()
                        .logoutUrl("/user/logout")
                        .logoutSuccessUrl("/user/home")
                        .invalidateHttpSession(true)
                        .addLogoutHandler(CookieClearingLogoutHandler())
            }
        }
    }
}
